@@ -1,22 +1,28 @@
-import logging
+from __future__ import annotations
 
 import ctranslate2
 import sentencepiece as spm
 import stanza
+from ctranslate2 import Translator
 
-from argostranslate import package, settings, models, sbd, apis, fewshot
-from argostranslate.utils import info, error
+from argostranslate import package, settings, sbd, apis, fewshot
+from argostranslate.models import ILanguageModel
+from argostranslate.package import Package
+from argostranslate.utils import info
 
 
 class Hypothesis:
     """Represents a translation hypothesis
 
     Attributes:
-        value (str): The hypothetical translation value
-        score (float): The score representing the quality of the translation
+        value: The hypothetical translation value
+        score: The score representing the quality of the translation
     """
 
-    def __init__(self, value, score):
+    value: str
+    score: float
+
+    def __init__(self, value: str, score: float):
         self.value = value
         self.score = score
 
@@ -31,61 +37,64 @@ class Hypothesis:
 
 
 class ITranslation:
-    """Respresents a translation between two Languages
+    """Represents a translation between two Languages
 
     Attributes:
-        from_lang (Language): The Language this Translation translates from.
-        to_lang (Language): The Language this Translation translates to.
+        from_lang: The Language this Translation translates from.
+        to_lang: The Language this Translation translates to.
 
     """
 
-    def translate(self, input_text):
+    from_lang: Language
+    to_lang: Language
+
+    def translate(self, input_text: str) -> str:
         """Translates a string from self.from_lang to self.to_lang
 
         Args:
-            input_text (str): The text to be translated.
+            input_text: The text to be translated.
 
         Returns:
-            str: input_text translated.
+            input_text translated.
 
         """
         return self.hypotheses(input_text, num_hypotheses=1)[0].value
 
-    def hypotheses(self, input_text, num_hypotheses=4):
+    def hypotheses(self, input_text: str, num_hypotheses: int = 4) -> list[Hypothesis]:
         """Translates a string from self.from_lang to self.to_lang
 
         Args:
-            input_text (str): The text to be translated.
-            num_hypotheses (int): Number of hypothetic results expected
+            input_text: The text to be translated.
+            num_hypotheses: Number of hypothetic results expected
 
         Returns:
-            [Hypothesis]: List of translation hypotheses
+            List of translation hypotheses
 
         """
         raise NotImplementedError()
 
     @staticmethod
-    def split_into_paragraphs(input_text):
+    def split_into_paragraphs(input_text: str) -> list[str]:
         """Splits input_text into paragraphs and returns a list of paragraphs.
 
         Args:
-            input_text (str): The text to be split.
+            input_text: The text to be split.
 
         Returns:
-            [str]: A list of paragraphs.
+            A list of paragraphs.
 
         """
         return input_text.split("\n")
 
     @staticmethod
-    def combine_paragraphs(paragraphs):
+    def combine_paragraphs(paragraphs: list[str]) -> str:
         """Combines a list of paragraphs together.
 
         Args:
-            paragraphs ([str]): A list of paragraphs.
+            paragraphs: A list of paragraphs.
 
         Returns:
-            [str]: list of n paragraphs combined into one string.
+            list of n paragraphs combined into one string.
 
         """
         return "\n".join(paragraphs)
@@ -101,16 +110,19 @@ class Language:
     """Represents a language that can be translated from/to.
 
     Attributes:
-        code (str): The code representing the language.
-        name (str): The human readable name of the language.
-        translations_from ([ITranslation]): A list of the translations
+        code: The code representing the language.
+        name: The human readable name of the language.
+        translations_from: A list of the translations
             that translate from this language.
-        translations_to ([ITranslation]): A list of the translations
+        translations_to: A list of the translations
             that translate to this language
 
     """
 
-    def __init__(self, code, name):
+    translations_from: list[ITranslation] = []
+    translations_to: list[ITranslation] = []
+
+    def __init__(self, code: str, name: str):
         self.code = code
         self.name = name
         self.translations_from = []
@@ -119,14 +131,14 @@ class Language:
     def __str__(self):
         return self.name
 
-    def get_translation(self, to):
+    def get_translation(self, to: Language) -> ITranslation | None:
         """Gets a translation from this Language to another Language.
 
         Args:
-            to (Language): The Language to look for a Translation to.
+            to: The Language to look for a Translation to.
 
         Returns:
-            ITranslation: A valid Translation if there is one in translations_from
+            A valid Translation if there is one in translations_from
                 else None.
 
         """
@@ -139,15 +151,15 @@ class Language:
 
 
 class PackageTranslation(ITranslation):
-    """Translation from a package"""
+    """A Translation that is installed with a package"""
 
-    def __init__(self, from_lang, to_lang, pkg):
+    def __init__(self, from_lang: Language, to_lang: Language, pkg: Package):
         self.from_lang = from_lang
         self.to_lang = to_lang
         self.pkg = pkg
         self.translator = None
 
-    def hypotheses(self, input_text, num_hypotheses):
+    def hypotheses(self, input_text: str, num_hypotheses: int = 4) -> list[Hypothesis]:
         if self.translator is None:
             model_path = str(self.pkg.package_path / "model")
             self.translator = ctranslate2.Translator(model_path, device=settings.device)
@@ -179,18 +191,18 @@ class PackageTranslation(ITranslation):
 class IdentityTranslation(ITranslation):
     """A Translation that doesn't modify input_text."""
 
-    def __init__(self, lang):
+    def __init__(self, lang: Language):
         """Creates an IdentityTranslation.
 
         Args:
-            lang (Language): The Language this Translation translates
+            lang: The Language this Translation translates
                 from and to.
 
         """
         self.from_lang = lang
         self.to_lang = lang
 
-    def hypotheses(self, input_text, num_hypotheses):
+    def hypotheses(self, input_text: str, num_hypotheses: int = 4):
         return [Hypothesis(input_text, 0) for i in range(num_hypotheses)]
 
 
@@ -198,25 +210,29 @@ class CompositeTranslation(ITranslation):
     """A ITranslation that is performed by chaining two Translations
 
     Attributes:
-        t1 (ITranslation): The first Translation to apply.
-        t2 (ITranslation): The second Translation to apply.
+        t1: The first Translation to apply.
+        t2: The second Translation to apply.
 
     """
 
-    def __init__(self, t1, t2):
+    t1: ITranslation
+    t2: ITranslation
+    from_lang: Language
+    to_lang: Language
+
+    def __init__(self, t1: ITranslation, t2: ITranslation):
         """Creates a CompositeTranslation.
 
         Args:
-            t1 (ITranslation): The first Translation to apply.
-            t2 (ITranslation): The second Translation to apply.
-
+            t1: The first Translation to apply.
+            t2: The second Translation to apply.
         """
         self.t1 = t1
         self.t2 = t2
         self.from_lang = t1.from_lang
         self.to_lang = t2.to_lang
 
-    def hypotheses(self, input_text, num_hypotheses):
+    def hypotheses(self, input_text: str, num_hypotheses: int = 4) -> list[Hypothesis]:
         t1_hypotheses = self.t1.hypotheses(input_text, num_hypotheses)
 
         # Combine hypotheses
@@ -247,19 +263,23 @@ class CachedTranslation(ITranslation):
 
     """
 
-    def __init__(self, underlying):
+    underlying: ITranslation
+    from_lang: Language
+    to_lang: Language
+    cache: dict
+
+    def __init__(self, underlying: ITranslation):
         """Creates a CachedTranslation.
 
         Args:
-            underlying (ITranslation): The underlying translation to cache.
-
+            underlying: The underlying translation to cache.
         """
         self.underlying = underlying
         self.from_lang = underlying.from_lang
         self.to_lang = underlying.to_lang
         self.cache = dict()
 
-    def hypotheses(self, input_text, num_hypotheses=4):
+    def hypotheses(self, input_text: str, num_hypotheses: int = 4) -> list[Hypothesis]:
         new_cache = dict()  # 'text': ['t1'...('tN')]
         paragraphs = ITranslation.split_into_paragraphs(input_text)
         translated_paragraphs = []
@@ -293,13 +313,18 @@ class CachedTranslation(ITranslation):
         return hypotheses_to_return
 
 
-class LibreTranslateTranslation(ITranslation):
-    def __init__(self, from_lang, to_lang, api):
+class RemoteTranslation(ITranslation):
+    """A translation provided by a remote LibreTranslate server"""
+
+    from_lang: Language
+    to_lang: Language
+
+    def __init__(self, from_lang: Language, to_lang: Language, api):
         self.from_lang = from_lang
         self.to_lang = to_lang
         self.api = api
 
-    def hypotheses(self, input_text, num_hypotheses=1):
+    def hypotheses(self, input_text: str, num_hypotheses: int = 1) -> list[Hypothesis]:
         """LibreTranslate only supports single hypotheses.
 
         A list of length num_hypotheses will be returned with identical hypotheses.
@@ -308,14 +333,25 @@ class LibreTranslateTranslation(ITranslation):
         return [Hypothesis(result, 0)] * num_hypotheses
 
 
+# Backwards compatibility, renamed in 1.8
+LibreTranslateTranslation = RemoteTranslation
+
+
 class FewShotTranslation(ITranslation):
-    # TODO: Document and handle detect language
-    def __init__(self, from_lang, to_lang, language_model):
+    """A translation performed with a few shot language model"""
+
+    from_lang: Language
+    to_lang: Language
+    language_model: ILanguageModel
+
+    def __init__(
+        self, from_lang: Language, to_lang: Language, language_model: ILanguageModel
+    ):
         self.from_lang = from_lang
         self.to_lang = to_lang
         self.language_model = language_model
 
-    def hypotheses(self, input_text, num_hypotheses=1):
+    def hypotheses(self, input_text: str, num_hypotheses: int = 1) -> list[Hypothesis]:
         # Split into sentences
         DEFAULT_SENTENCE_LENGTH = 250
         sentences = []
@@ -355,21 +391,23 @@ class FewShotTranslation(ITranslation):
         return [Hypothesis(to_return, 0)] * num_hypotheses
 
 
-def apply_packaged_translation(pkg, input_text, translator, num_hypotheses=4):
+def apply_packaged_translation(
+    pkg: Package, input_text: str, translator: Translator, num_hypotheses: int = 4
+) -> list[Hypothesis]:
     """Applies the translation in pkg to translate input_text.
 
     Args:
-        pkg (Package): The package that provides the translation.
-        input_text (str): The text to be translated.
-        translator (ctranslate2.Translator): The CTranslate2 Translator
-        num_hypotheses (int): The number of hypotheses to generate
+        pkg: The package that provides the translation.
+        input_text: The text to be translated.
+        translator: The CTranslate2 Translator
+        num_hypotheses: The number of hypotheses to generate
 
     Returns:
-        [Hypothesis]: A list of Hypothesis's for translating input_text
+        A list of Hypothesis's for translating input_text
 
     """
 
-    info("apply_packaged_translation")
+    info("apply_packaged_translation", input_text)
 
     # Sentence boundary detection
     if pkg.type == "sbd":
@@ -408,13 +446,13 @@ def apply_packaged_translation(pkg, input_text, translator, num_hypotheses=4):
             info("sbd_index", sbd_index)
             info(input_text[start_index:sbd_index])
             start_index = sbd_index
-    info("sentences:", sentences)
+    info("sentences", sentences)
 
     # Tokenization
     sp_model_path = str(pkg.package_path / "sentencepiece.model")
     sp_processor = spm.SentencePieceProcessor(model_file=sp_model_path)
     tokenized = [sp_processor.encode(sentence, out_type=str) for sentence in sentences]
-    info("tokenized:", tokenized)
+    info("tokenized", tokenized)
 
     # Translation
     BATCH_SIZE = 32
@@ -450,7 +488,7 @@ def apply_packaged_translation(pkg, input_text, translator, num_hypotheses=4):
     return value_hypotheses
 
 
-def get_installed_languages():
+def get_installed_languages() -> list[Language]:
     """Returns a list of Languages installed from packages"""
 
     info("get_installed_languages")
@@ -555,6 +593,55 @@ def get_installed_languages():
     return languages
 
 
-def load_installed_languages():
+def load_installed_languages() -> list[Language]:
     """Deprecated 1.2, use get_installed_languages"""
     return get_installed_languages()
+
+
+def get_language_from_code(code: str) -> Language | None:
+    """Gets a language object from a code
+
+    An exception will be thrown if an installed language with this
+    code can not be found.
+
+    Args:
+        code: The ISO 639 code of the language
+
+    Returns:
+        The language object
+    """
+    return next(list(filter(lambda x: x.code == code, get_installed_languages())), None)
+    
+
+
+def get_translation_from_codes(from_code: str, to_code: str) -> ITranslation:
+    """Gets a translation object from codes for from and to languages
+
+    An exception will be thrown if an installed translation between the from lang
+    and to lang can not be found.
+
+    Args:
+        from_code: The ISO 639 code of the source language
+        to_code: The ISO 639 code of the target language
+
+    Returns:
+        The translation object
+    """
+    from_lang = get_language_from_code(from_code)
+    to_lang = get_language_from_code(to_code)
+    return from_lang.get_translation(to_lang)
+
+
+def translate(q: str, from_code: str, to_code: str) -> str:
+    """Translate a string of text
+
+    Args:
+        q: The text to translate
+        from_code: The ISO 639 code of the source language
+        to_code: The ISO 639 code of the target language
+
+    Returns:
+        The translated text
+    """
+    translation = get_translation_from_codes(from_code, to_code)
+    return translation.translate(q)
